<?php

namespace App\Http\Controllers;

use App\Models\Order;
use App\Models\UsersCentros;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;


class OrderController extends Controller
{

    //---------------------- Función que devuelve un dataset con los pedidos de un determinado día
    public function ordersFecha(Request $request)
   {
    //Obtenemos la fecha y la formateamos
    $now=Carbon::createFromFormat('Y-m-d', $request->post("fecha"));
    $dia=$now->day;
    $mes=$now->month;
    $anio=$now->year;
    $fecha=$anio.'-'.str_pad($mes, 2, "0", STR_PAD_LEFT).'-'.str_pad($dia, 2, "0", STR_PAD_LEFT);
    $sig=$now->addDays(1);
 
    //Calculamos el día siguiente a la fecha recibida
    $dia2=$sig->day;
    $mes2=$sig->month;
    $anio2=$sig->year;
    $fecha2=$anio2.'-'.str_pad($mes2, 2, "0", STR_PAD_LEFT).'-'.str_pad($dia2, 2, "0", STR_PAD_LEFT);

    //Realizamos la consulta de los pedidos entre ambas fechas, ordenado por centro, estado y fecha de entrega (desc)
    $orders=Order::whereBetween("fecha_entrega_desde",array(
        Carbon::createFromFormat('Y-m-d H:i:s',$fecha.' 00:00:00'),
        Carbon::createFromFormat('Y-m-d H:i:s',$fecha2.' 00:00:00')
    ))->orderBy("centro.cod_centro", 'asc')->orderBy("estado", 'asc')->orderBy('fecha_entrega_desde', 'desc')->get();

    $aux_centro=0;
    //Recorremos el dataset recibido para calcular de cada centro, los pedidos totales del día y los pedneintes de entregar
    foreach($orders as &$order){
        if($aux_centro <> $order->centro["cod_centro"] ){
            $aux_centro=$order->centro["cod_centro"] ;

            //Cuando hay un cambio de centro al recorrer el dataset, consultamos el total de pedidos
            $orders_totales=Order::where("centro.cod_centro","=",$order->centro["cod_centro"])
            ->whereBetween("fecha_entrega_desde",array(
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha.' 00:00:00'),
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha2.' 00:00:00')))
            ->count();

            //Consultamos el total de pedidos entregados (estado=1)
            $orders_entregados=Order::where("centro.cod_centro","=",$order->centro["cod_centro"])
            ->where("estado","=",1)
            ->whereBetween("fecha_entrega_desde",array(
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha.' 00:00:00'),
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha2.' 00:00:00')))
            ->count();

            $order["pedidos_totales"]=$orders_totales;
            $order["pedidos_entregados"]=$orders_entregados;
        }
        else{
            $order["pedidos_totales"]=$orders_totales;
            $order["pedidos_entregados"]=$orders_entregados;
        }
    }

    //Devolvemos el dataset a la vista blade 'pedidos'
    return view('pedidos',['orders' => $orders,'fecha'=>$fecha]);
   }

   //--------------- Funcion que muestra los pedidos del día actual cuando accedemos a ventana de pedidos
    public function index(Request $request)
   { //Obtnenemos la fecha de hoy y la del dia siguiente
    $fec = $request->fec ;

   if($fec==''){
       $now = Carbon::now();
   }
   else{
       $now = Carbon::createFromFormat('Y-m-d H:i:s',$fec.' 00:00:00');
   }
   
   //Formateamos las fechas
   $dia=$now->day;
   $mes=$now->month;
   $anio=$now->year;
   $fecha=$anio.'-'.str_pad($mes, 2, "0", STR_PAD_LEFT).'-'.str_pad($dia, 2, "0", STR_PAD_LEFT);
   $sig=$now->addDays(1);

   $dia2=$sig->day;
   $mes2=$sig->month;
   $anio2=$sig->year;
   $fecha2=$anio2.'-'.str_pad($mes2, 2, "0", STR_PAD_LEFT).'-'.str_pad($dia2, 2, "0", STR_PAD_LEFT);
   
    //Realizamos la consulta de los pedidos entre ambas fechas, ordenado por centro, estado y fecha de entrega (desc)
    $orders=Order::whereBetween("fecha_entrega_desde",array(
        Carbon::createFromFormat('Y-m-d H:i:s',$fecha.' 00:00:00'),
        Carbon::createFromFormat('Y-m-d H:i:s',$fecha2.' 00:00:00')
    ))->orderBy("centro.cod_centro", 'asc')->orderBy("estado", 'asc')->orderBy('fecha_entrega_desde', 'desc')->get();
    
    $aux_centro=0;
    $aux_order=0;
    //Recorremos el dataset recibido para calcular de cada centro, los pedidos totales del día y los pedneintes de entregar
    foreach($orders as &$order){
        if($aux_centro <> $order->centro["cod_centro"] ){
            $aux_centro=$order->centro["cod_centro"] ;

            //Comprobar si está en una de los centros asociados al usuario... para ordenar los pedidos por los centros a los que el usuario pertenezca
            $userCentro=UsersCentros::where("centro","=",$aux_centro)
            ->where("email","=",Auth::user()->email)->get();
            
            if(isset($userCentro[0]->order)){
                $aux_order=$userCentro[0]->order;
            }
            else
            {
                $aux_order=0;
            }
            
            //Cuando hay un cambio de centro al recorrer el dataset, consultamos el total de pedidos
            $orders_totales=Order::where("centro.cod_centro","=",$order->centro["cod_centro"])
            ->whereBetween("fecha_entrega_desde",array(
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha.' 00:00:00'),
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha2.' 00:00:00')))
            ->count();

            //Consultamos el total de pedidos entregados (estado=1)
            $orders_entregados=Order::where("centro.cod_centro","=",$order->centro["cod_centro"])
            ->where("estado","=",1)
            ->whereBetween("fecha_entrega_desde",array(
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha.' 00:00:00'),
            Carbon::createFromFormat('Y-m-d H:i:s',$fecha2.' 00:00:00')))
            ->count();

            $order["pedidos_totales"]=$orders_totales;
            $order["pedidos_entregados"]=$orders_entregados;
            $order["orden"]=$aux_order;
        }
        else{
            $order["pedidos_totales"]=$orders_totales;
            $order["pedidos_entregados"]=$orders_entregados;
            $order["orden"]=$aux_order;
        }
    }

    //Reordenamos el dataset
    $orders = $orders->sortBy([
        ['orden', 'desc'],
        ['centro.cod_centro', 'asc'],
        ['estado', 'asc'],
        ['fecha_entrega_desde', 'desc'],                
    ]);

    //Redirigimos a la vista
    return view('pedidos',['orders' => $orders,'fecha'=>$fecha]);
   }

   //--------------- Funcion que actualiza el estado de un pedido
   public function actualizarEstado(Request $request)
   {
        //Obtenemos el pedido a modificar
        $pedido=Order::findOrFail($request->id);

        //Cambiamos el estado del pedido según corresponda (de entregado a pendiente o viceversa)
        if($request->input('estado')==1){
            $pedido->estado=0;            
        }else{
            $pedido->estado=1;
        }

        //Almacenamos que usuario a modificado el pedido y en qué momento
        $pedido->usuario=Auth::user()->email;
        $pedido->updated_at=Carbon::now();
        //Guardamos los datos
        $pedido->save();
        //Redirecionamos a la vista de pedidos
        return redirect()->route('orders',['centro' => $request->input('centro'),'fec' => $request->input('fecha')]); 

   }


   //Función de apoyo que nos sirve para ordenar un array por los criterios que se establezcan. En nuestro caso, el dataset de pedidos
   function phparraysort($Array, $SortBy=array(), $Sort = SORT_REGULAR) {
        if (is_array($Array) && count($Array) > 0 && !empty($SortBy)) {
            $Map = array();                     
            foreach ($Array as $Key => $Val) {
                $Sort_key = '';                         
                foreach ($SortBy as $Key_key) {
                        $Sort_key .= $Val[$Key_key];
                }                
                $Map[$Key] = $Sort_key;
            }
            asort($Map, $Sort);
            $Sorted = array();
            foreach ($Map as $Key => $Val) {
                $Sorted[] = $Array[$Key];
            }
            return array_reverse($Sorted);
        }
        return $Array;
    }   

}
