<?php

namespace App\Http\Controllers\admin;

use App\Models\Order;
use App\Models\Centro;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use Carbon\CarbonPeriod;
use App\Http\Controllers\Controller;
use MongoDB\BSON\UTCDateTime;
use MongoDB\Client;


//--- CONTROLADOR DE ADMINISTRACION DE PEDIDOS ---
class OrderAdminController extends Controller
{


    public function index()
   {

    //Al entrar en la página de inicio mostramos todos los pedidos existentes en el sistema, paginando 50 por pagina
    $orders=Order::where("nombre_fichero","<>","''")->orderBy('fecha_entrega_desde', 'desc')->orderBy("centro.cod_centro", 'asc')->orderBy('hora_entrega_desde', 'desc')->paginate(50);
    $centros=Centro::All();

    return view('admin/pedidos',['orders' => $orders,'centros'=>$centros]);
   }

   //Función que muestra los pedidos permitiendo introducir como criterios el centro o dos fechas determinadas(desde/hasta)
   public function pedidosFecha(Request $request)
   {
    //Obtenemos los datos de centro y de fecha recibida
    $cen=$request->post("centro");  
    $fec=$request->post("fecha");
    $fec2=$request->post("fecha2");

    //Si hemos recibido fecha...
    if($request->post("fecha")<>''){
        //Si el centro es ditinto de 'Todos'
        if($request->post("centro")<>'Todos'){
            //Obtenemos los pedidos de dicho centro entre las fechas seleccionadas
            $orders=Order::where("centro._id","=",$request->post("centro"))
            ->whereBetween("fecha_entrega_desde",array(
                Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha").' 00:00:00'),
                Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha2").' 00:00:00')->addDays(1))
                )->orderBy("centro.cod_centro", 'asc')->orderBy('fecha_entrega_desde', 'desc')->orderBy('hora_entrega_desde', 'desc')->get();
        }//Si se han seleccionado todos los centros...
        else{ // Obtenemos los pedidos existentes entre las fechas seleccionadas en todos los centros
            $orders=Order::whereBetween("fecha_entrega_desde",array(
                Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha").' 00:00:00'),
                Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha2").' 00:00:00')->addDays(1))
                )->orderBy("centro.cod_centro", 'asc')->orderBy('fecha_entrega_desde', 'desc')->orderBy('hora_entrega_desde', 'desc')->get();
        }
    }//Si no se ha recibido fecha
    else{ //Si se ha especificado un centro...
        if($request->post("centro")<>'Todos'){
            //Obtenemos los pedidos de un centro concreto
            $orders=Order::where("centro._id","=",$request->post("centro"))
            ->orderBy("centro.cod_centro", 'asc')->orderBy('fecha_entrega_desde', 'desc')->orderBy('hora_entrega_desde', 'desc')->get();
        }//Si se han seleccionado todos los centros
        else{ //Obtenemos todos los pedidos del sistema
            $orders=Order::where("nombre_fichero","<>","''")->orderBy("centro.cod_centro", 'asc')->orderBy('fecha_entrega_desde', 'desc')->orderBy('hora_entrega_desde', 'desc')->get();
        }
    }

    //Obtenemos toddos los centros
    $centros=Centro::All();

    $aux_centro=0;
    //Recorremos el dataset de pedidos recibidos para actualizar los datos de pedidos totales y entregados
    foreach($orders as &$order){
        if($aux_centro <> $order->centro["cod_centro"] ){
            $aux_centro=$order->centro["cod_centro"] ;

            //Calculamos los pedidos totales para el centro
            $orders_totales=Order::where("centro.cod_centro","=",$order->centro["cod_centro"])
            ->whereBetween("fecha_entrega_desde",array(
            Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha").' 00:00:00'),
            Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha2").' 00:00:00')->addDays(1)))
            ->count();

            //Calculamos los pedidos entregados para el centro
            $orders_entregados=Order::where("centro.cod_centro","=",$order->centro["cod_centro"])
            ->where("estado","=",1)
            ->whereBetween("fecha_entrega_desde",array(
            Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha").' 00:00:00'),
            Carbon::createFromFormat('Y-m-d H:i:s',$request->post("fecha2").' 00:00:00')->addDays(1)))
            ->count();

            //Actualizamos los datos en el dataset
            $order["pedidos_totales"]=$orders_totales;
            $order["pedidos_entregados"]=$orders_entregados;
        }
        else{
            $order["pedidos_totales"]=$orders_totales;
            $order["pedidos_entregados"]=$orders_entregados;
        }
    }    
    
    //Redireccionamos a la vista
    return view('admin/pedidosfecha',['orders' => $orders,'centros'=>$centros,'cen'=>$cen,'fec'=>$fec,'fec2'=>$fec2]);
   }

   public function selecfechas(Request $request)
   {
    return view('admin/selecfechas');
   }

   //Obtenemos el cuadro de facturación entre las fechas recibidas
   public function pedidosMes(Request $request)
   {
    //Obtenemos las fechas de desde y hasta
    $fec=$request->post("fecha");
    $fec2=$request->post("fecha2");
      
    //Formateamos las fechas
    $period=CarbonPeriod::create($fec,$fec2);
    $desde_fecha=Carbon::createFromFormat('Y-m-d H:i:s',$fec.' 00:00:00');
    $hasta_fecha=Carbon::createFromFormat('Y-m-d H:i:s',$fec2.' 00:00:00');    

    //Realizamos una agregacion en mongodb para obtener agrupados los datos por día  y centro
    $orders=Order::raw()->aggregate([['$match' => ['fecha_entrega_desde' => ['$gte' => new UTCDateTime($desde_fecha), '$lt' => new UTCDateTime($hasta_fecha->addDay())]]], ['$project' => ['_id' => 0, 'centro.cod_centro' => 1, 'centro.desc_centro' => 1, 'dia' => ['$concat' => [['$convert' => ['input' => ['$dayOfMonth' => '$fecha_entrega_hasta'], 'to' => 'string']], '/', ['$convert' => ['input' => ['$month' => '$fecha_entrega_hasta'], 'to' => 'string']]]]]], ['$group' => ['_id' => ['cod_centro' => '$centro.cod_centro', 'desc_centro' => '$centro.desc_centro', 'dia' => '$dia'], 'entregas' => ['$sum' => 1]]], ['$sort' => ['_id.cod_centro' => 1, '_id.dia' => 1]]]);
    
    
    //Convertimos el periodo en un aray de fechas
    $dates = $period->toArray();

    //Creamos un array asociativo donde para cada centro, asignamos el vector de días que corresponda
    $aux_centro="";

    //Recorremos dia por dia e inicializamos el vector de pedidos por dia a 0
    foreach ($period as $key => $date) {
        $vector[$date->format('d/m')]=0;
    }
    $sw=0;
    $dias=$vector;
    $suma=array();
    
    //Rellenamos el array del cuadro de facturación recorriendo los datos de la agregación
    foreach ($orders as $order){
        $sw=1;
        $centro=$order->_id["cod_centro"].' - '.$order->_id["desc_centro"];
        if($aux_centro <> $centro){
            //Si se produce un cambio de centro se acutaliza el array del cuadro  con el vector de dias y se inicializa a 0 de nuevo
            if($aux_centro !== ""){
                $cuadro[$aux_centro]=$dias;
                $dias=$vector;
            }
            $aux_centro=$centro;            
        }
          //Obtenemos los datos de la agregación
          $aux_dia=$order->_id["dia"];
          $idx=strpos($aux_dia,"/");
          $d=str_pad(substr($aux_dia,0,$idx),2,'0',STR_PAD_LEFT);
          $m=str_pad(substr($aux_dia,$idx+1),2,'0',STR_PAD_LEFT);
        
          //Asignamos el valor de entregas al vector de días del centro
          $dias[$d.'/'.$m]=$order->entregas;
          //Actualizamos los totales
          if(array_key_exists($d.'/'.$m, $suma)){
            $suma[$d.'/'.$m]+=$order->entregas;
          }else{
            $suma[$d.'/'.$m]=$order->entregas;
          }
          
    }
    if($sw==1){
        $cuadro[$centro]=$dias;
    };
    if(!isset($cuadro)){
        $cuadro=array();
    }
    //Redirigimos a la vista
    return view('admin/pedidosmes',['cuadro' => $cuadro,'period' => $period,'desde_fecha'=>$fec,'hasta_fecha'=>$fec2,'totales' =>$suma]);
   }
          
}
