/*Copyright (C) 2023  José Luis Rubio Leira

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>*/

package com.example.blanizapedidos;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ProgressBar;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity {
    WebView webPagina;  //Componente para visualizar la página web
    ProgressBar progressBar;  //Barras de progreso para mostrar la carga de la pagina

    //Método que configura e inicia el webview con la URL indicada
    private void startWebView(String url) {

       WebSettings settings = webPagina.getSettings(); //Obtenemos los settings del webview

        settings.setJavaScriptEnabled(true);         //Activamos el uso de javascript
        webPagina.setScrollBarStyle(View.SCROLLBARS_OUTSIDE_OVERLAY);  //Estilo de la barra de desplazamiento

        settings.setBuiltInZoomControls(true);   //Activamos los botones de zoom
        settings.setUseWideViewPort(true);  //Compatibilidad con viewport amplios
        webPagina.setWebChromeClient(new WebChromeClient()); //estabecemos que el webclient sea un ChromeClient
        settings.setLoadWithOverviewMode(true); //Ajusta la visualización si el contenido es más ancho que la pantalla
        settings.setDomStorageEnabled(true); //Almacenamiento DOM activado

        //Establecemos el cliente de webview
        webPagina.setWebViewClient(new WebViewClient() {
            @Override
            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                if (url.startsWith("tel:")) {  //Si se accede a una URL que empiece por tel: accedemos al marcador telefonico
                    Intent intent = new Intent(Intent.ACTION_DIAL, Uri.parse(url));
                    startActivity(intent);
                    view.reload();
                    return true;
                }

                //Cargamos la URL en el webview
                view.loadUrl(url);
                return true;
            }

            //Mostramos la barra de progreso al inicio de la carga de la página
            @Override
            public void onPageStarted(WebView view, String url, Bitmap favicon) {
                super.onPageStarted(view, url, favicon);
                progressBar.setVisibility(View.VISIBLE);
            }

            //Ocultamos la barra de progreso al finalizar la carga de la página
            @Override
            public void onPageFinished(WebView view, String url) {
                super.onPageFinished(view, url);
                progressBar.setVisibility(View.GONE);
            }

            //En caso de error de carga, mostramos un mensaje Toast
            @Override
            public void onReceivedError(WebView view, int errorCode, String description, String failingUrl) {
                Toast.makeText(MainActivity.this, "Error:" + description, Toast.LENGTH_SHORT).show();

            }


        });
        //Cargamos la URL en el webview
        webPagina.loadUrl(url);
    }

    //Escondemos las barras del sistema
    private void hideSystemUI(View mDecorView) {
        // Set the IMMERSIVE flag.
        // Set the content to appear under the system bars so that the content
        // doesn't resize when the system bars hide and show.
        mDecorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION // hide nav bar
                        | View.SYSTEM_UI_FLAG_FULLSCREEN // hide status bar
                        | View.SYSTEM_UI_FLAG_IMMERSIVE);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //Obtenemos acceso a los componentes del activity (webview y progressbar)
        webPagina=(WebView) findViewById(R.id.webPagina);
        progressBar=(ProgressBar) findViewById(R.id.progressBar);
        webPagina.clearCache(true);  //Vaciamos la caché

        hideSystemUI(webPagina);

        //Iniciamos el webview apuntando a la URL de la página de pedidos
        startWebView("http://27.0.173.180/orders");
    }
}