/* Copyright (C) 2023  José Luis Rubio Leira

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
const { isSchema } = require('joi');
const Joi=require('joi');
const mongoose=require('mongoose');
const {centroSchema} = require('./centro');

//Creamos un esquema de mongoose para los pedidos
const orderSchema = new mongoose.Schema({
    
    tipo_movimiento: {
        type:String,
         required:true,
         maxlength:1
    },
    centro: {
        type:centroSchema,
        required:true,
    },
    id_reparto: {
        type:String,
         required:true,
         maxlength:10
    },
    fecha_entrega_desde: {
        type:Date,
         required:true,
    },
    fecha_entrega_hasta: {
        type:Date,
         required:true
    },
    fecha_generacion_ticket: {
        type:Date,
         required:true
    },
    nombre_cliente: {
        type:String,
        maxlength:50
    },
    apellidos_cliente: {
        type:String,
        maxlength:100
    },
    tipo_via_nombre: {
        type:String,
        maxlength:145
    },
    portal_num_esc_piso_mano: {
        type:String,
         maxlength:45
    },
    codigo_postal: {
        type:String,
         maxlength:5
    },
    localidad_municipio: {
        type:String,
         maxlength:100
    },
    telefono_cliente: {
        type:String,
        maxlength:9
    },
    identificativo_cliente: {
        type:String,
         maxlength:9
    },
    ascensor_cliente: {
        type:String,
         maxlength:1
    },
    info_adicional: {
        type:String,
        maxlength:50
    },
    cajas_seco: {
        type:Number,
         required:true,
         min:0,
         max:99
    },
    cajas_fresco: {
        type:Number,
         required:true,
         min:0,
         max:99    
    },
    cajas_congelado: {
        type:Number,
         required:true,
         min:0,
         max:99
    },
    bolsas_fresco: {
        type:Number,
         required:true,
         min:0,
         max:99
    },
    bolsas_congelado: {
        type:Number,
         required:true,
         min:0,
         max:99
    },
    importe_ticket: {
        type:Number,
        required:true
    },
    tipo_cliente: {
        type:String,
        maxlength:1
    },
    nombre_fichero: {
        type:String,
        required:true,
        maxlength:255
    },
    fecha_carga: {
        type:Date,
         required:true,
         default:Date.now
    },
    estado: {
        type:String,
        maxlength:255
    }

});

//Creamos el modelo basandonos en el esquema
const Order=mongoose.model('Order',orderSchema);

//Validamos los datos del pedido con Joi
function validateOrder(order){
    const schema=Joi.object({
        tipo_movimiento:Joi.string().max(1).required(),
        centro:Joi.object().required().allow(),
        id_reparto:Joi.string().max(10).required(),
        fecha_entrega_desde:Joi.date().required(),
        fecha_entrega_hasta:Joi.date().required(),        
        fecha_generacion_ticket:Joi.date().required(),
        nombre_cliente:Joi.string().min(0).max(50),        
        apellidos_cliente:Joi.string().min(0).max(100),        
        tipo_via_nombre:Joi.string().min(0).max(145),        
        portal_num_esc_piso_mano:Joi.string().min(0).max(45),
        codigo_postal:Joi.string().min(0).max(5),        
        localidad_municipio:Joi.string().min(0).max(100),
        telefono_cliente:Joi.string().min(0).max(9),
        identificativo_cliente:Joi.string().min(0).max(9),
        ascensor_cliente:Joi.string().min(0).max(1),
        info_adicional:Joi.string().min(0).max(50),
        cajas_seco:Joi.number().min(0).max(99),
        cajas_fresco:Joi.number().min(0).max(99),
        cajas_congelado:Joi.number().min(0).max(99),
        bolsas_fresco:Joi.number().min(0).max(99),
        bolsas_congelado:Joi.number().min(0).max(99),
        importe_ticket:Joi.number(),
        tipo_cliente:Joi.string().min(0).max(1),
        nombre_fichero:Joi.string().max(255).required(),
        fecha_carga: Joi.date().required(),
        estado:Joi.string().max(255)

    });
    
    return schema.validate(order);
}

module.exports.Order=Order;
module.exports.validate=validateOrder;
module.exports.orderSchema=orderSchema;
