<?php

namespace Tests\Unit;

use App\Http\Controllers\LoginController;
use Illuminate\Http\Request;
use Tests\TestCase;
use App\Models\User;

class LoginControllerTest extends TestCase
{
    
    public function testIndexConAutenticacion()
    {

        // Creamos un usuario usando la factoría de usuarios
        $user = User::factory()->create();
        $user->email='jlrubiol@gmail.com';        
    
        // Simulamos que el usuario está autenticado
        $this->actingAs($user);
    
        // Visita la ruta asociada al método 'index' de la clase
        $response = $this->get('/login');
    
        // Comprobamos  que se redirige a la ruta esperada, en este caso /orders
        $response->assertRedirect('/orders');
    }   

    public function testIndexSinAutenticacion()
    {
        // Creamos un usuario usando la factoría de usuarios
        $user = User::factory()->create();
    
        // Visita la ruta asociada al método 'index' de la clase
        $response = $this->get('/login');
    
        // Comprobamos que la respuesta es correcta
        $response->assertStatus(200);

        // Comprobamos  que se redirige a la vista de login
        $response->assertViewIs('login');
      
    }    

    public function testLoginOK()
    {
        // Creamos un usuario usando la factoría de usuarios con la contraseña pruebasuoc
        $user = User::factory()->create(['password' => bcrypt($password = 'pruebasuoc')]);

        // Enviar una solicitud POST a la función incluyendo los datos necesarios que pase de pendiente a entregado.
        $request = new Request();
        $request->setMethod('POST');
        $request->request->add([
            'email' => $user->email,
            'password' => 'pruebasuoc',
        ]); 

        // Simulamos una sesión
        $request->setLaravelSession($this->app['session.store']);
        
        //Realizamos la llamada al método del controlador
        $controller = new LoginController();        
        $response=$controller->validarLogin($request);        
    
        // Comprobamos que el usuario autenticado es el que ha de ser
        $this->assertAuthenticatedAs($user);
        
        //Borramos los usuarios creados
        User::where('email', 'like', '%example%')->delete();        
    }
    
    public function testLoginKO()
    {
        // Creamos un usuario usando la factoría de usuarios con la contraseña pruebasuoc
        $user = User::factory()->create(['password' => bcrypt($password = 'pruebasuoc')]);

        // Enviar una solicitud POST a la función incluyendo los datos necesarios que pase de pendiente a entregado.
        $request = new Request();
        $request->setMethod('POST');
        $request->request->add([
            'email' => $user->email,
            'password' => 'passwordErronea',
        ]); 

        // Simulamos una sesión
        $request->setLaravelSession($this->app['session.store']);

        //Realizamos la llamada al método del controlador
        $controller = new LoginController();        
        $response=$controller->validarLogin($request);        
        
        // Comprobamos que el usuario no fue autenticado
        $this->assertGuest();

        //Limpiamos los usuarios creados
        User::where('email', 'like', '%example%')->delete();
    }


}
